/* IMPORT */
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const config_1 = require("./config");
/* CONSTS */
const Consts = {
    languageId: 'todo',
    symbols: {
        project: ':',
        box: config_1.default.getKey('symbols.box'),
        done: config_1.default.getKey('symbols.done'),
        cancel: config_1.default.getKey('symbols.cancel'),
        tag: '@'
    },
    colors: {
        cancel: config_1.default.getKey('colors.cancel'),
        done: config_1.default.getKey('colors.done'),
        code: config_1.default.getKey('colors.code'),
        comment: config_1.default.getKey('colors.comment'),
        project: config_1.default.getKey('colors.project'),
        tag: config_1.default.getKey('colors.tag')
    },
    tags: {
        names: config_1.default.getKey('tags.names'),
        backgroundColors: config_1.default.getKey('tags.backgroundColors'),
        foregroundColors: config_1.default.getKey('tags.foregroundColors')
    },
    regexes: {
        impossible: /(?=a)b/,
        todo: /^[^\S\n]*((?!--|––|——)(?:[-❍❑■⬜□☐▪▫–—≡→›✘xX✔✓☑+]|\[[ xX+-]?\])[^\n]*)/,
        todoToken: /^[^\S\n]*(?!--|––|——)([-❍❑■⬜□☐▪▫–—≡→›✘xX✔✓☑+]|\[[ xX+-]?\])/,
        todoBox: /^[^\S\n]*((?!--|––|——)(?:[-❍❑■⬜□☐▪▫–—≡→›]|\[ ?\])[^\n]*)/,
        todoCancel: /^[^\S\n]*((?!--|––|——)(?:[✘xX]|\[-\])[^\n]*)/,
        todoDone: /^[^\S\n]*((?:[✔✓☑+]|\[[xX+]\])[^\n]*)/,
        project: /^(?![^\S\n]*(?!--|––|——)(?:[-❍❑■⬜□☐▪▫–—≡→›✘xX✔✓☑+]|\[[ xX+-]?\])[^\n]*)[^\S\n]*(.+:[^\S\n]*)(?:(?=@(?!.+ +[^@]))|$)/,
        archive: /^(?![^\S\n]*(?!--|––|——)(?:[-❍❑■⬜□☐▪▫–—≡→›✘xX✔✓☑+]|\[[ xX+-]?\])[^\n]*)([^\S\n]*Archive:.*$)/,
        comment: /^(?![^\S\n]*(?!--|––|——)(?:[-❍❑■⬜□☐▪▫–—≡→›✘xX✔✓☑+]|\[[ xX+-]?\])[^\n]*)(?![^\S\n]*.+:[^\S\n]*(?:(?=@(?!.+ +[^@]))|$))[^\S\n]*([^\n]+)/,
        tag: /(?:^|[^a-zA-Z0-9`])(@[^\s*~(]+(?:\([^)]*\))?)/,
        tagSpecial: /(?=a)b/,
        tagStarted: /@started\(([^)]*)\)/,
        tagFinished: /@(?:done|cancelled)\(([^)]*)\)/,
        tagElapsed: /@(?:lasted|wasted)\(([^)]*)\)/,
        code: /((?:```[\s\S]*?```)|(?:`[^`\n]*`))/,
        bold: /(?:^|\s)(\*.+\*)(?:\s|$)/,
        italic: /(?:^|\s)(_.+_)(?:\s|$)/,
        strikethrough: /(?:^|\s)(~.+~)(?:\s|$)/
    }
};
if (Consts.tags.names.length) {
    Consts.regexes.tagSpecial = new RegExp(`(?:^|[^a-zA-Z0-9])(@(?:${Consts.tags.names.map(n => _.escapeRegExp(n)).join('|')})(?:\\([^)]*\\))?)(?![a-zA-Z])`);
}
const archiveName = config_1.default.getKey('archive.name');
if (archiveName) {
    Consts.regexes.archive = new RegExp(Consts.regexes.archive.source.replace('Archive', _.escapeRegExp(archiveName)));
}
/* EXPORT */
exports.default = Consts;
//# sourceMappingURL=consts.js.map