/* IMPORT */
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const vscode = require("vscode");
const utils_1 = require("../../utils");
/* LINE */
class Line {
    constructor() {
        this.TYPES = [];
    }
    /* RANGE */
    getRange(textLine, startCharacter, endCharacter) {
        if (startCharacter < 0 || endCharacter < 0)
            return;
        const line = textLine.range.start.line;
        return new vscode.Range(line, startCharacter, line, endCharacter);
    }
    getRangesRegex(textLine, posRegex, negRegex, negRange) {
        function rangesDifference(pos, neg, length) {
            const cells = _.fill(Array(length), false);
            pos.forEach(({ start, end }) => _.fill(cells, true, start, end));
            neg.forEach(({ start, end }) => _.fill(cells, false, start, end));
            const ranges = [];
            let start = null, end = null;
            for (let i = 0, l = cells.length; i < l; i++) {
                const cell = cells[i];
                const asd = textLine.text[i];
                if (start === null) {
                    if (cell)
                        start = i;
                }
                else {
                    if (!cell)
                        end = i;
                }
                if (start !== null && (end !== null || i === l - 1)) {
                    end = end !== null ? end : l;
                    ranges.push({ start, end });
                    start = null;
                    end = null;
                }
            }
            return ranges;
        }
        const posMatches = utils_1.default.getAllMatches(textLine.text, posRegex), posRanges = utils_1.default.matches2ranges(posMatches);
        if (!posRanges.length)
            return []; // Early exit //TSC
        let ranges = posRanges, negRanges = _.castArray(negRange || []).filter(range => range.line === textLine.lineNumber);
        if (negRegex) {
            _.castArray(negRegex).forEach(negRegex => {
                const negMatches = utils_1.default.getAllMatches(textLine.text, negRegex);
                negRanges = negRanges.concat(utils_1.default.matches2ranges(negMatches));
            });
        }
        if (negRanges.length) {
            ranges = rangesDifference(ranges, negRanges, textLine.text.length);
        }
        return ranges.map(({ start, end }) => this.getRange(textLine, start, end));
    }
    /* ITEMS */
    getRanges(items, negRange) {
        const ranges = items.map(item => this.getItemRanges(item, negRange)), //TSC
        zipped = _.zip(...ranges), compact = zipped.map(_.compact), concat = compact.map(r => _.concat([], ...r));
        return concat;
    }
    getItemRanges(item, negRange) {
        return [item.range];
    }
    getDecorations(items, negRange) {
        let ranges = this.getRanges(items, negRange);
        return this.TYPES.map((type, index) => ({
            type,
            ranges: ranges[index] || []
        }));
    }
}
/* EXPORT */
exports.default = Line;
//# sourceMappingURL=line.js.map