/* IMPORT */
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const line_1 = require("./items/line");
const code_1 = require("./items/code");
const comment_1 = require("./items/comment");
const project_1 = require("./items/project");
const todo_1 = require("./items/todo");
const consts_1 = require("../consts");
const utils_1 = require("../utils");
/* DOCUMENT */
class Document {
    constructor(textDocument) {
        this.textDocument = textDocument;
    }
    /* GET */
    getItems(Item, regex) {
        const matches = utils_1.default.getAllMatches(this.textDocument.getText(), regex);
        return matches.map(match => {
            const range = utils_1.default.match2range(match), startPos = this.textDocument.positionAt(range.start), endPos = this.textDocument.positionAt(range.end), startLine = this.textDocument.lineAt(startPos), endLine = this.textDocument.lineAt(endPos), text = _.last(match);
            return new Item(this.textDocument, startPos, endPos, startLine, endLine, text);
        });
    }
    getItemAt(Item, lineNumber, checkValidity = true) {
        const line = this.textDocument.lineAt(lineNumber);
        if (checkValidity && !Item.is(line.text))
            return;
        return new Item(this.textDocument, line.range.start, line.range.end, line, line, line.text);
    }
    getLines() {
        return _.range(this.textDocument.lineCount)
            .map(lineNr => this.textDocument.lineAt(lineNr))
            .map(line => new line_1.default(this.textDocument, line.range.start, line.range.end, line, line, line.text));
    }
    getCodes() {
        return this.getItems(code_1.default, consts_1.default.regexes.code);
    }
    getCodeAt(lineNumber, checkValidity) {
        return this.getItemAt(code_1.default, lineNumber, checkValidity);
    }
    getComments() {
        return this.getItems(comment_1.default, consts_1.default.regexes.comment);
    }
    getCommentAt(lineNumber, checkValidity) {
        return this.getItemAt(comment_1.default, lineNumber, checkValidity);
    }
    getProjects() {
        return this.getItems(project_1.default, consts_1.default.regexes.project);
    }
    getProjectAt(lineNumber, checkValidity) {
        return this.getItemAt(project_1.default, lineNumber, checkValidity);
    }
    getTodos() {
        return this.getItems(todo_1.default, consts_1.default.regexes.todo);
    }
    getTodoAt(lineNumber, checkValidity) {
        return this.getItemAt(todo_1.default, lineNumber, checkValidity);
    }
}
/* EXPORT */
exports.default = Document;
//# sourceMappingURL=document.js.map