/* IMPORT */
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const moment = require("moment");
require("moment-precise-range-plugin");
const item_1 = require("./item");
const config_1 = require("../../config");
const consts_1 = require("../../consts");
const utils_1 = require("../../utils");
/* TODO */
class Todo extends item_1.default {
    /* HELPERS */
    getStatus() {
        const box = this.isBox(), done = !box && this.isDone(), cancelled = !box && !done && this.isCancelled(), other = !box && !done && !cancelled;
        return { box, done, cancelled, other };
    }
    setToken(replacement, startIndex, endIndex = startIndex) {
        const was = this.getStatus();
        this.text = `${this.text.substring(0, startIndex)}${replacement}${this.text.substring(endIndex)}`;
        const is = this.getStatus();
        if (((was.done || was.cancelled) && is.box) || (!was.other && is.other)) {
            return this.unfinish();
        }
        else if ((was.box && !is.box) || (was.cancelled && is.done) || (was.done && is.cancelled) || (was.other && (is.done || is.cancelled))) {
            return this.finish();
        }
        else {
            return this.makeEdit();
        }
    }
    makeEdit() {
        if (this.startLine.text === this.text)
            return;
        return utils_1.default.editor.makeReplaceEdit(this.startLine.lineNumber, this.text, this.startPos.character, this.endPos.character);
    }
    /* TAGS */
    addTag(tag) {
        this.text = `${_.trimEnd(this.text)} ${tag}`;
        return this.makeEdit();
    }
    removeTag(tagRegex) {
        this.text = _.trimEnd(this.text.replace(tagRegex, ''));
        return this.makeEdit();
    }
    replaceTag(tagRegex, tag) {
        this.removeTag(tagRegex);
        return this.addTag(tag);
    }
    /* TIMEKEEPING */
    start() {
        const date = moment(), format = config_1.default.getKey('timekeeping.started.format'), timestamp = date.format(format), tag = `@started(${timestamp})`;
        return this.replaceTag(consts_1.default.regexes.tagStarted, tag);
    }
    unstart() {
        return this.removeTag(consts_1.default.regexes.tagStarted);
    }
    finish() {
        const started = this.text.match(consts_1.default.regexes.tagStarted);
        if (config_1.default.getKey('timekeeping.finished.enabled') || started) {
            this.unfinish();
            const isPositive = this.isDone();
            /* FINISH */
            const finishedDate = moment(), finishedFormat = config_1.default.getKey('timekeeping.finished.format'), finishedTimestamp = finishedDate.format(finishedFormat), finishedTag = `@${isPositive ? 'done' : 'cancelled'}(${finishedTimestamp})`;
            this.addTag(finishedTag);
            /* ELAPSED */
            if (config_1.default.getKey('timekeeping.elapsed.enabled') && started) {
                const startedTimestamp = _.last(started), startedFormat = config_1.default.getKey('timekeeping.started.format'), startedDate = moment(startedTimestamp, startedFormat), diff = moment['preciseDiff'](startedDate, finishedDate), elapsedTag = `@${isPositive ? 'lasted' : 'wasted'}(${diff})`;
                return this.addTag(elapsedTag);
            }
        }
        return this.makeEdit();
    }
    unfinish() {
        this.removeTag(consts_1.default.regexes.tagFinished);
        return this.removeTag(consts_1.default.regexes.tagElapsed);
    }
    /* TOKENS */
    toggleToken(token, removeToken, insertToken, force) {
        const tokenMatch = this.text.match(new RegExp(`^[^\\S\\n]*(${_.escapeRegExp(token)})`));
        if (tokenMatch) {
            if (force === true)
                return;
            const endIndex = tokenMatch.index + tokenMatch[0].length, startIndex = endIndex - tokenMatch[1].length, spaceNr = !removeToken && this.text.length >= (endIndex + 1) && this.text[endIndex].match(/\s/) ? 1 : 0;
            return this.setToken(removeToken, startIndex, endIndex + spaceNr);
        }
        const otherMatch = this.text.match(consts_1.default.regexes.todoToken);
        if (otherMatch) {
            if (force === false)
                return;
            const endIndex = otherMatch.index + otherMatch[0].length, startIndex = endIndex - otherMatch[1].length;
            return this.setToken(token, startIndex, endIndex);
        }
        if (insertToken) {
            if (force === false)
                return;
            let startIndex = this.text.search(/\S/);
            if (startIndex === -1)
                startIndex = this.text.length;
            return this.setToken(`${insertToken} `, startIndex);
        }
    }
    toggleBox(force) {
        return this.toggleToken(consts_1.default.symbols.box, '', consts_1.default.symbols.box, force);
    }
    box() {
        return this.toggleBox(true);
    }
    unbox() {
        return this.toggleBox(false);
    }
    toggleCancel(force) {
        return this.toggleToken(consts_1.default.symbols.cancel, consts_1.default.symbols.box, consts_1.default.symbols.cancel, force);
    }
    cancel() {
        return this.toggleCancel(true);
    }
    uncancel() {
        return this.toggleCancel(false);
    }
    toggleDone(force) {
        return this.toggleToken(consts_1.default.symbols.done, consts_1.default.symbols.box, consts_1.default.symbols.done, force);
    }
    done() {
        return this.toggleDone(true);
    }
    undone() {
        return this.toggleDone(false);
    }
    /* IS */
    isBox() {
        return item_1.default.is(this.text, consts_1.default.regexes.todoBox);
    }
    isDone() {
        return item_1.default.is(this.text, consts_1.default.regexes.todoDone);
    }
    isCancelled() {
        return item_1.default.is(this.text, consts_1.default.regexes.todoCancel);
    }
    static is(str) {
        return super.is(str, consts_1.default.regexes.todo);
    }
}
/* EXPORT */
exports.default = Todo;
//# sourceMappingURL=todo.js.map